@@ -160,7 +160,7 @@ gst_ffmpegmux_get_replacement (const char *name)
     {"mp3", "id3v2mux"},
     {"mp2", "id3v2mux"}
   };
-  int i;
+  guint i;
 
   for (i = 0; i < sizeof (blacklist) / sizeof (blacklist[0]); i++) {
     if (strcmp (blacklist[i].name, name) == 0) {
@@ -446,10 +446,10 @@ gst_ffmpegmux_request_new_pad (GstElement * element,
   /* AVStream needs to be created */
   st = avformat_new_stream (ffmpegmux->context, NULL);
   st->id = collect_pad->padnum;
-  st->codec->codec_type = type;
-  st->codec->codec_id = AV_CODEC_ID_NONE;       /* this is a check afterwards */
-  st->codec->bit_rate = bitrate;
-  st->codec->frame_size = framesize;
+  st->codecpar->codec_type = type;
+  st->codecpar->codec_id = AV_CODEC_ID_NONE;    /* this is a check afterwards */
+  st->codecpar->bit_rate = bitrate;
+  st->codecpar->frame_size = framesize;
   /* we fill in codec during capsnego */
 
   /* we love debug output (c) (tm) (r) */
@@ -475,6 +475,7 @@ gst_ffmpegmux_setcaps (GstPad * pad, GstCaps * caps)
   GstFFMpegMux *ffmpegmux = (GstFFMpegMux *) (gst_pad_get_parent (pad));
   GstFFMpegMuxPad *collect_pad;
   AVStream *st;
+  AVCodecContext tmp;
 
   collect_pad = (GstFFMpegMuxPad *) gst_pad_get_element_private (pad);
 
@@ -484,12 +485,14 @@ gst_ffmpegmux_setcaps (GstPad * pad, GstCaps * caps)
 
   /* for the format-specific guesses, we'll go to
    * our famous codec mapper */
-  if (gst_ffmpeg_caps_to_codecid (caps, st->codec) == AV_CODEC_ID_NONE)
+  if (gst_ffmpeg_caps_to_codecid (caps, &tmp) == AV_CODEC_ID_NONE)
     goto not_accepted;
 
+  avcodec_parameters_from_context (st->codecpar, &tmp);
+
   /* copy over the aspect ratios, ffmpeg expects the stream aspect to match the
    * codec aspect. */
-  st->sample_aspect_ratio = st->codec->sample_aspect_ratio;
+  st->sample_aspect_ratio = st->codecpar->sample_aspect_ratio;
 
   GST_LOG_OBJECT (pad, "accepted caps %" GST_PTR_FORMAT, caps);
   return TRUE;
@@ -563,23 +566,23 @@ gst_ffmpegmux_collected (GstCollectPads * pads, gpointer user_data)
       AVStream *st = ffmpegmux->context->streams[collect_pad->padnum];
 
       /* check whether the pad has successfully completed capsnego */
-      if (st->codec->codec_id == AV_CODEC_ID_NONE) {
+      if (st->codecpar->codec_id == AV_CODEC_ID_NONE) {
         GST_ELEMENT_ERROR (ffmpegmux, CORE, NEGOTIATION, (NULL),
             ("no caps set on stream %d (%s)", collect_pad->padnum,
-                (st->codec->codec_type == AVMEDIA_TYPE_VIDEO) ?
+                (st->codecpar->codec_type == AVMEDIA_TYPE_VIDEO) ?
                 "video" : "audio"));
         return GST_FLOW_ERROR;
       }
       /* set framerate for audio */
-      if (st->codec->codec_type == AVMEDIA_TYPE_AUDIO) {
-        switch (st->codec->codec_id) {
+      if (st->codecpar->codec_type == AVMEDIA_TYPE_AUDIO) {
+        switch (st->codecpar->codec_id) {
           case AV_CODEC_ID_PCM_S16LE:
           case AV_CODEC_ID_PCM_S16BE:
           case AV_CODEC_ID_PCM_U16LE:
           case AV_CODEC_ID_PCM_U16BE:
           case AV_CODEC_ID_PCM_S8:
           case AV_CODEC_ID_PCM_U8:
-            st->codec->frame_size = 1;
+            st->codecpar->frame_size = 1;
             break;
           default:
           {
@@ -590,8 +593,8 @@ gst_ffmpegmux_collected (GstCollectPads * pads, gpointer user_data)
             buffer = gst_collect_pads_peek (ffmpegmux->collect,
                 (GstCollectData *) collect_pad);
             if (buffer) {
-              st->codec->frame_size =
-                  st->codec->sample_rate *
+              st->codecpar->frame_size =
+                  st->codecpar->sample_rate *
                   GST_BUFFER_DURATION (buffer) / GST_SECOND;
               gst_buffer_unref (buffer);
             }
@@ -726,15 +729,12 @@ gst_ffmpegmux_collected (GstCollectPads * pads, gpointer user_data)
   if (best_pad != NULL) {
     GstBuffer *buf;
     AVPacket pkt;
-    gboolean need_free = FALSE;
     GstMapInfo map;
 
     /* push out current buffer */
     buf =
         gst_collect_pads_pop (ffmpegmux->collect, (GstCollectData *) best_pad);
 
-    ffmpegmux->context->streams[best_pad->padnum]->codec->frame_number++;
-
     /* set time */
     pkt.pts = gst_ffmpeg_time_gst_to_ff (GST_BUFFER_TIMESTAMP (buf),
         ffmpegmux->context->streams[best_pad->padnum]->time_base);
@@ -757,11 +757,7 @@ gst_ffmpegmux_collected (GstCollectPads * pads, gpointer user_data)
     else
       pkt.duration = 0;
     av_write_frame (ffmpegmux->context, &pkt);
-    if (need_free) {
-      g_free (pkt.data);
-    } else {
-      gst_buffer_unmap (buf, &map);
-    }
+    gst_buffer_unmap (buf, &map);
     gst_buffer_unref (buf);
   } else {
     /* close down */
@@ -844,7 +840,7 @@ gst_ffmpeg_mux_simple_caps_set_int_list (GstCaps * caps, const gchar * field,
 {
   GValue list = { 0, };
   GValue val = { 0, };
-  gint i;
+  guint i;
 
   g_return_if_fail (GST_CAPS_IS_SIMPLE (caps));
 
@@ -880,13 +876,12 @@ gst_ffmpegmux_register (GstPlugin * plugin)
     NULL, NULL, NULL
   };
   GType type;
-  AVOutputFormat *in_plugin;
-
-  in_plugin = av_oformat_next (NULL);
+  const AVOutputFormat *in_plugin;
+  void *i = 0;
 
   GST_LOG ("Registering muxers");
 
-  while (in_plugin) {
+  while ((in_plugin = av_muxer_iterate (&i))) {
     gchar *type_name;
     GstRank rank = GST_RANK_MARGINAL;
 
@@ -925,12 +920,12 @@ gst_ffmpegmux_register (GstPlugin * plugin)
         !strncmp (in_plugin->name, "webm", 4)
         ) {
       GST_LOG ("Ignoring muxer %s", in_plugin->name);
-      goto next;
+      continue;
     }
 
     if ((!strncmp (in_plugin->long_name, "raw ", 4))) {
       GST_LOG ("Ignoring raw muxer %s", in_plugin->name);
-      goto next;
+      continue;
     }
 
     if (gst_ffmpegmux_get_replacement (in_plugin->name))
@@ -958,9 +953,6 @@ gst_ffmpegmux_register (GstPlugin * plugin)
     }
 
     g_free (type_name);
-
-  next:
-    in_plugin = av_oformat_next (in_plugin);
   }
 
   GST_LOG ("Finished registering muxers");
