@@ -23,7 +23,7 @@
 #include "libavutil/x86/w64xmmtest.h"
 
 wrap(avcodec_open2(AVCodecContext *avctx,
-                   AVCodec *codec,
+                   const AVCodec *codec,
                    AVDictionary **options))
 {
     testxmmclobbers(avcodec_open2, avctx, codec, options);
@@ -83,9 +83,9 @@ wrap(avcodec_send_packet(AVCodecContext *avctx, const AVPacket *avpkt))
     testxmmclobbers(avcodec_send_packet, avctx, avpkt);
 }
 
-wrap(avcodec_receive_frame(AVCodecContext *avctx, AVFrame *frame))
+wrap(avcodec_receive_packet(AVCodecContext *avctx, AVPacket *avpkt))
 {
-    testxmmclobbers(avcodec_receive_frame, avctx, frame);
+    testxmmclobbers(avcodec_receive_packet, avctx, avpkt);
 }
 
 wrap(avcodec_send_frame(AVCodecContext *avctx, const AVFrame *frame))
@@ -93,7 +93,7 @@ wrap(avcodec_send_frame(AVCodecContext *avctx, const AVFrame *frame))
     testxmmclobbers(avcodec_send_frame, avctx, frame);
 }
 
-wrap(avcodec_receive_packet(AVCodecContext *avctx, AVPacket *avpkt))
+wrap(avcodec_receive_frame(AVCodecContext *avctx, AVFrame *frame))
 {
-    testxmmclobbers(avcodec_receive_packet, avctx, avpkt);
+    testxmmclobbers(avcodec_receive_frame, avctx, frame);
 }
